﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace WindowsFormsApplication1
{
    public partial class mainForm : Form
    {
        SerialPort commPort;
        bool done;

        delegate void AppendCallback(string text);
        delegate void EnableCallback(bool text);

        
        public mainForm()
        {
            InitializeComponent();
        }

        private void append_text( string txt ) 
        {
            textOut.Text += txt;
            clearButton.Enabled = true;
        }

        private void enable_action(bool en)
        {
            actionButton.Enabled = en;
            cancelButton.Enabled = !en;
        }

        private void copy_comport_to_scrn()
        {
            done = false;
            int read = 0;
            char[] buffer = new char[256];
            do
            {
                if ( commPort.BytesToRead == 0 )
                {
                    Thread.Sleep(250);
                    continue;
                }
                read = commPort.Read(buffer, 0, buffer.Length);
                String data = new string(buffer, 0, read);
                this.Invoke(new AppendCallback(append_text), new object[] { data.Replace("\n", "\r\n") });
            }
            while ( !done );
        }

        private void serial_thread_delegate()
        {
            try
            {
                copy_comport_to_scrn();
                commPort.Close();
            }
            catch
            {
                this.Invoke(new AppendCallback(append_text), new object[] { "\r\n*** comm port disconnected ***\r\n" });
            }
            finally
            {
                EnableCallback d = new EnableCallback(enable_action);
                this.Invoke(d, new object[] { true });
            }
        }

        private void actionButton_Click(object sender, EventArgs e)
        {
            commPort.PortName = portList.SelectedItem.ToString();
            commPort.Open();
            actionButton.Enabled = false;
            cancelButton.Enabled = true;
            Thread t = new Thread(new ThreadStart(this.serial_thread_delegate));
            t.Start();
        }

        private void mainForm_Load(object sender, EventArgs e)
        {
            portList.Items.AddRange(SerialPort.GetPortNames());
            portList.SelectedIndex = 0;
            commPort = new SerialPort();
            actionButton.Enabled = true;
            cancelButton.Enabled = false;
            clearButton.Enabled = false;
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            done = true;
        }

        private void clearButton_Click(object sender, EventArgs e)
        {
            textOut.Text = "";
            clearButton.Enabled = false;
        }
    }
}
